
#ifndef THREAD_PTHREAD_H_INCLUDED
#define THREAD_PTHREAD_H_INCLUDED

#include <pthread.h>
typedef pthread_t yarv_thread_id_t;
typedef pthread_mutex_t yarv_thread_lock_t;

#define native_mutex_lock   pthread_mutex_lock
#define native_mutex_unlock pthread_mutex_unlock

#endif /* THREAD_PTHREAD_H_INCLUDED */


/**********************************************************************/

#ifdef THREAD_SYSTEM_DEPENDENT_IMPLEMENTATION

#define native_mutex_initialize(lock) do { \
  pthread_mutex_t _lock = PTHREAD_MUTEX_INITIALIZER; \
  ((*lock) = _lock); \
} while (0)

#define native_cleanup_push pthread_cleanup_push
#define native_cleanup_pop  pthread_cleanup_pop
#define native_thread_join  pthread_join

static void
Init_native_thread(){
  /* */
}

static int system_working = 1;

void
native_thread_cleanup(yarv_vm_t *vm){
  system_working = 0;
}

NOINLINE(static int thread_start_func_2(yarv_thread_t *th, VALUE *stack_start));
void static thread_cleanup_func(void *th_ptr);

static void *
thread_start_func_1(void *th_ptr)
{
  yarv_thread_t *th = th_ptr;
  VALUE stack_start;
  /* ignore self and klass */
  
  native_cleanup_push(thread_cleanup_func, th);

  /* run */
  thread_start_func_2(th, &stack_start);
  thread_cleanup_func(th);
  native_cleanup_pop(0);
  return 0;
}

static void make_timer_thread();

static int
native_thread_crteate(yarv_thread_t *th)
{
  pthread_attr_t attr;
  size_t stack_size = 4 * 1024 - sizeof(int); /* 4KB */
  int err;
  static int init = 0;

  if(!init){
    make_timer_thread();
  }
  
  thread_debug("create: %p, stack size: %ld\n", th, stack_size);

  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr, stack_size);
  err = pthread_create(&th->thread_id, &attr, thread_start_func_1, th);
  if(err != 0){
    th->status = THREAD_KILLED;
    rb_raise(rb_eThreadError, "can't create Thread (%d)", err);
  }
  return err;
}

static pthread_t time_thread;

static void*
thread_timer(void *dummy)
{
  while(system_working){
#ifdef HAVE_NANOSLEEP
    struct timespec req, rem;

    req.tv_sec = 0;
    req.tv_nsec = 10 * 1000; /* 10 ms */
    nanosleep(&req, &rem);
#else
    struct timeval tv;
    tv.tv_sec = 0;
    tv.tv_usec = 10000;
    select(0, NULL, NULL, NULL, &tv);
#endif
    GET_VM()->interrupt_flag = 1;
  }
  return NULL;
}

static void
make_timer_thread()
{
  if(!time_thread){
    pthread_create(&time_thread, 0, thread_timer, 0);
  }
}

#endif /* THREAD_SYSTEM_DEPENDENT_IMPLEMENTATION */

